//-----------------------------------------------------------------------------
//    KAmodCOMBO library
//-----------------------------------------------------------------------------
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//-----------------------------------------------------------------------------



#include "KAmodCOMBO.h"
#include <util/delay.h>


// STLED316_Initialize
// Configures I/O lines and INT1 interrupt 

void STLED316_Initialize()
{
	// Configure input/output lines
	STLED316_CLK_DD |= (1 << STLED316_CLK_PIN);
	STLED316_STB_DD |= (1 << STLED316_STB_PIN);
	STLED316_DIO_DD |= (1 << STLED316_DIO_PIN);
	STLED316_IRQ_DD &= ~(1 << STLED316_IRQ_PIN);

	// Set high level on comm. lines
	STLED316_CLK_PORT |= (1 << STLED316_CLK_PIN);	
	STLED316_STB_PORT |= (1 << STLED316_STB_PIN);
	STLED316_DIO_PORT |= (1 << STLED316_DIO_PIN);

	// Trigger INT1 for falling edge only
	MCUCR &= ~0x4;
	MCUCR |= (0x8);

	// Enable INT1 interrupt
	GIMSK |= (1<<INT1);
}

void STLED316_Write(unsigned char * Buffer, unsigned char BytesToWrite)
{
	unsigned char BitNum, ByteNum;
	STLED316_DIO_DD |= (1 << STLED316_DIO_PIN);

	STLED316_STB_PORT &= ~(1 << STLED316_STB_PIN);
	_delay_us(1);

	for (ByteNum=0; ByteNum < BytesToWrite; ByteNum++)
	{
		for (BitNum = 0; BitNum < 8; BitNum++ )
		{
			STLED316_CLK_PORT &= ~(1 << STLED316_CLK_PIN);
			if ( Buffer[ByteNum] & (1 << BitNum) )
				STLED316_DIO_PORT |= (1 << STLED316_DIO_PIN); else STLED316_DIO_PORT &= ~(1 << STLED316_DIO_PIN);
			STLED316_CLK_PORT |= (1 << STLED316_CLK_PIN);
			_delay_us(1);
		}
	}
	_delay_us(1);
	STLED316_STB_PORT |= (1 << STLED316_STB_PIN);
}

void STLED316_Read(unsigned char Command, unsigned char * Buffer, char BytesToRead)
{
	unsigned char BitNum, ByteNum;
	//Send command

	STLED316_DIO_DD |= (1 << STLED316_DIO_PIN);

	STLED316_STB_PORT &= ~(1 << STLED316_STB_PIN);

	for (BitNum = 0; BitNum < 8; BitNum++ )
	{
		STLED316_CLK_PORT &= ~(1 << STLED316_CLK_PIN);
		if ( Command & (1 << BitNum) )
			STLED316_DIO_PORT |= (1 << STLED316_DIO_PIN); else STLED316_DIO_PORT &= ~(1 << STLED316_DIO_PIN);
		STLED316_CLK_PORT |= (1 << STLED316_CLK_PIN);
		_delay_us(1);
	}

	
	//Read data
	STLED316_DIO_DD &= ~(1 << STLED316_DIO_PIN);

	for (ByteNum=0; ByteNum < BytesToRead; ByteNum++)
	{
		Buffer[ByteNum] = 0;
		for (BitNum = 0; BitNum < 8; BitNum++ )
		{
			STLED316_CLK_PORT &= ~(1 << STLED316_CLK_PIN);
			_delay_us(1);
			STLED316_CLK_PORT |= (1 << STLED316_CLK_PIN);
			_delay_us(1);
			if (STLED316_DIO_PINS & (1 << STLED316_DIO_PIN)) Buffer[ByteNum] |= (1 << BitNum);
						
		}
	}

	STLED316_STB_PORT |= (1 << STLED316_STB_PIN);
}


// STLED316_DisplayOn
// This function turns display on, you MUST call it before STLED316 can display anything.

void STLED316_DisplayOn()
{
	unsigned char Buffer = 0xD;
	STLED316_Write(&Buffer, 1);
}


// STLED316_DisplayOff
// Turns display off.
void STLED316_DisplayOff()
{
	unsigned char Buffer = 0xE;
	STLED316_Write(&Buffer, 1);
}


// STLED316_SetLEDStates
// This function turns on/off LEDs depending on LEDs variable bits, bit 0 = 0 turns off D1, bit 0 = 1 turns on D1,
// bit 1 = 0 turns off D2, bit 1 = 1 turns on D2 and so on.
void STLED316_SetLEDStates(unsigned char LEDs)
{
	unsigned char Buffer[2] = {0x8, LEDs};
	STLED316_Write(Buffer, 2);
}


// STLED316_SetBrghtCtrlModeAndDigitCnt
// Set brightness control mode
// Mode:
//   - STLED316_BRGHT_MODE_CONST - constant mode, all segments and LEDs will have equal brightness
//     passed in ConstBrightness  parameter (values 0..7)
//   - STLED316_BRGHT_MODE_VAR - brightness can be set individually
// DigitCount: 1..6 - number of enabled displays
void STLED316_SetBrghtCtrlModeAndDigitCnt(unsigned char Mode, unsigned char ConstBrightness, unsigned char DigitCount)
{	
    unsigned char Buffer[2] = {0x10, Mode | ((DigitCount > 0) & (DigitCount < 7) ? (DigitCount - 1) : 5) | ((ConstBrightness & 0x7) << 5)};
	STLED316_Write(Buffer, 2);
}



// STLED316_SetLEDBrightness
// LEDBrightness - 8-element array, every element is brightness of single LED, you should pass
// values in range 0..7. Do not expect that 0 turns off LED, it will be just less bright.

void STLED316_SetLEDBrightness(unsigned char * LEDBrightness)
{
	unsigned char Buffer[5], i;
	Buffer[0]=0x18;
	for (i=0; i<4; i++) Buffer[i+1] = (LEDBrightness[2*i] & 0x7) | ((LEDBrightness[2*i+1] & 0x7) << 4);
	STLED316_Write(Buffer, 5);
}


// STLED316_7Seg_Write
// This functions sets what LED display segments will be on.
// - DisplayNumber - number of display, values 0..5.
// - Segments - which segments will be on, combination of STLED316_xxx_SEG constants.
//   For example STLED316_RT_SEG | STLED316_RB_SEG gives '1' (both right segments on).
void STLED316_7Seg_Write(unsigned char DisplayNumber, unsigned char Segments)
{			
	unsigned char Buffer[2] = {DisplayNumber, Segments};
	STLED316_Write(Buffer, 2);
}


// STLED316_7Seg_SetBrghtInd
// Set brightness of two of 7-segment displays: 1st and 2nd, 3rd and 4th, 5th and 6th
// - DisplayNum - number of display: for DisplayNum=0 or DisplayNum=1 brightness of both displays 0 and 1 will be set,
//   for DisplayNum=2 or DisplayNum=3 brightness of both displays 2 and 3 will be set and so on.
// - Brightness1, Brightness2 - brightness values (0..7)
//
// To use this function you need to call STLED316_SetBrghtCtrlModeAndDigitCnt(STLED316_BRGHT_MODE_VAR... before!!!

void STLED316_7Seg_SetBrghtInd(unsigned char DisplayNum, unsigned char Brightness1, unsigned char Brightness2)
{
	unsigned char Buffer[2];
	Buffer[0]=0x10 | (((DisplayNum >> 1) & 0x3) + 1);
	Buffer[1]=(Brightness1 & 0x7) | ((Brightness2 & 0x7) <<4);
	STLED316_Write(Buffer, 2);
}


// STLED316_ReadLEDsAndButtons
// Reads LED states (into LEDs) and buttons states (buttons S1..S8 into Bunnons1 variable, buttons S9..S16 into Bunnons2 variable)
void STLED316_ReadLEDsAndButtons(unsigned char * LEDs, unsigned char * Buttons1, unsigned char * Buttons2)
{
	unsigned char Buffer[3];
	STLED316_Read(0x48, Buffer, 3);
	*LEDs = Buffer[0];
	*Buttons1 = Buffer[1];
	*Buttons2 = Buffer[2];
}


// STLED316_SetLEDBrightness
// LEDBrightness - 8-element array, every element is brightness of single LED, you should pass
// values in range 0..7. Do not expect that 0 turns off LED, it will be just less bright.
void STLED316_ReadLEDBrightness(unsigned char * LEDBrightness)
{
	int i;
	STLED316_Read(0x58, LEDBrightness, 4);

	for (i=3; i>=0; i--) 
	{
		LEDBrightness[2*i+1] = (LEDBrightness[i] >> 4) & 0x7;
		LEDBrightness[2*i] = LEDBrightness[i] & 0x7;	
	}
}
